@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2018 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #include "GmshConfig.h"
 #include <string>
@@ -31,23 +31,23 @@
 #include "OptHomFastCurving.h"
 #endif
 
-static void change_completeness_cb(Fl_Widget *w, void *data)
-{
-  highOrderToolsWindow *o = FlGui::instance()->highordertools;
-  bool onlyVisible = (bool)o->butt[1]->value();
-  if (!o->complete){
-    // BOF BOF BOF -- CG
-    SetHighOrderComplete(GModel::current(), onlyVisible);
-    o->complete = 1;
-  }
-  else if (o->complete){
-    // BOF BOF BOF -- CG
-    SetHighOrderInComplete(GModel::current(), onlyVisible);
-    o->complete = 0;
-  }
-  CTX::instance()->mesh.changed |= (ENT_LINE | ENT_SURFACE | ENT_VOLUME);
-  drawContext::global()->draw();
-}
+// static void change_completeness_cb(Fl_Widget *w, void *data)
+//{
+//  highOrderToolsWindow *o = FlGui::instance()->highordertools;
+//  bool onlyVisible = (bool)o->butt[1]->value();
+//  if (!o->complete){
+//    // BOF BOF BOF -- CG
+//    SetHighOrderComplete(GModel::current(), onlyVisible);
+//    o->complete = 1;
+//  }
+//  else if (o->complete){
+//    // BOF BOF BOF -- CG
+//    SetHighOrderIncomplete(GModel::current(), onlyVisible);
+//    o->complete = 0;
+//  }
+//  CTX::instance()->mesh.changed |= (ENT_CURVE | ENT_SURFACE | ENT_VOLUME);
+//  drawContext::global()->draw();
+//}
 
 static void highordertools_runp_cb(Fl_Widget *w, void *data)
 {
@@ -58,7 +58,7 @@ static void highordertools_runp_cb(Fl_Widget *w, void *data)
   bool incomplete = (bool)o->butt[0]->value();
   bool onlyVisible = (bool)o->butt[1]->value();
 
-  if (order == 1)
+  if(order == 1)
     SetOrder1(GModel::current());
   else
     SetOrderN(GModel::current(), order, linear, incomplete, onlyVisible);
@@ -73,7 +73,7 @@ static void highordertools_runp_cb(Fl_Widget *w, void *data)
   }
   */
 
-  CTX::instance()->mesh.changed |= (ENT_LINE | ENT_SURFACE | ENT_VOLUME);
+  CTX::instance()->mesh.changed |= (ENT_CURVE | ENT_SURFACE | ENT_VOLUME);
   drawContext::global()->draw();
 }
 
@@ -83,97 +83,148 @@ static void chooseopti_cb(Fl_Widget *w, void *data)
   const int algo = o->choice[2]->value();
 
   switch(algo) {
-  case 0: {                                           // Optimization
-    o->push[0]->activate();
+  case 0: { // Optimization
     o->choice[0]->activate();
     o->choice[3]->activate();
-    for (int i=2;i<=11;i++) o->value[i]->activate();
-    //    o->push[1]->activate();
+    o->value[1]->activate();
+    o->value[2]->activate();
+    o->value[3]->activate();
+    o->value[4]->activate();
+    o->value[5]->activate();
+    o->value[7]->activate();
+    o->value[8]->activate();
+    if(o->choice[3]->value() == 0) {
+      o->value[9]->deactivate();
+      o->value[10]->deactivate();
+      o->value[11]->deactivate();
+    }
+    else {
+      o->value[9]->activate();
+      o->value[10]->activate();
+      o->value[11]->activate();
+    }
     break;
   }
-  case 1: {                                           // Elastic analogy
+  case 1: { // Elastic analogy
     o->choice[0]->deactivate();
     o->choice[3]->deactivate();
-    for (int i=2;i<=11;i++) o->value[i]->deactivate();
-    //   o->push[1]->deactivate();
+    o->value[1]->deactivate();
+    o->value[2]->deactivate();
+    o->value[3]->deactivate();
+    o->value[4]->deactivate();
+    o->value[5]->deactivate();
+    o->value[7]->deactivate();
+    o->value[8]->deactivate();
+    o->value[9]->deactivate();
+    o->value[10]->deactivate();
+    o->value[11]->deactivate();
     break;
   }
-  case 2: {                                           // Fast curving
+  case 2: { // Fast curving
     o->choice[0]->deactivate();
     o->choice[3]->deactivate();
-    for (int i=2;i<=6;i++)
-      o->value[i]->deactivate();
-    for (int i=9;i<=11;i++) o->value[i]->deactivate();
-    for (int i=7;i<=8;i++) o->value[i]->activate();
-    //   o->push[1]->deactivate();
+    o->value[1]->deactivate();
+    o->value[2]->deactivate();
+    o->value[3]->deactivate();
+    o->value[4]->deactivate();
+    o->value[5]->deactivate();
+    o->value[7]->deactivate();
+    o->value[8]->deactivate();
+    o->value[9]->deactivate();
+    o->value[10]->deactivate();
+    o->value[11]->deactivate();
+    break;
+  }
+  case 3: { // Fast curving 2
+    o->choice[0]->deactivate();
+    o->choice[3]->deactivate();
+    o->value[1]->deactivate();
+    o->value[2]->deactivate();
+    o->value[3]->deactivate();
+    o->value[4]->deactivate();
+    o->value[5]->deactivate();
+    o->value[7]->deactivate();
+    o->value[8]->deactivate();
+    o->value[9]->deactivate();
+    o->value[10]->deactivate();
+    o->value[11]->deactivate();
     break;
   }
   }
-
 }
 
 static void chooseopti_strategy(Fl_Widget *w, void *data)
 {
   highOrderToolsWindow *o = FlGui::instance()->highordertools;
-  if (o->choice[3]->value() == 0) for (int i=9;i<=11;i++) o->value[i]->deactivate();
-  else for (int i=9;i<=11;i++) o->value[i]->activate();
+  if(o->choice[3]->value() == 1)
+    for(int i = 9; i <= 11; i++) o->value[i]->activate();
+  else
+    for(int i = 9; i <= 11; i++) o->value[i]->deactivate();
 }
 
 static void highordertools_runopti_cb(Fl_Widget *w, void *data)
 {
   highOrderToolsWindow *o = FlGui::instance()->highordertools;
 
-  if(o->butt[3]->value())
-    FlGui::instance()->graph[0]->showMessages();
+  if(o->butt[3]->value()) FlGui::instance()->graph[0]->showMessages();
 
   const int algo = o->choice[2]->value();
-  double threshold_min = o->value[1]->value();
   bool onlyVisible = (bool)o->butt[1]->value();
-  int nbLayers = (int) o->value[2]->value();
-  double threshold_max = o->value[8]->value();
 
   int NE = 0;
-  for (GModel::riter it = GModel::current()->firstRegion(); it != GModel::current()->lastRegion(); ++it){
+  for(GModel::riter it = GModel::current()->firstRegion();
+      it != GModel::current()->lastRegion(); ++it) {
     NE += (*it)->getNumMeshElements();
   }
-
+  int dim = GModel::current()->getDim() == 3 ? (NE ? 3 : 2) :
+                                               GModel::current()->getDim();
 
 #if defined(HAVE_OPTHOM)
   switch(algo) {
-  case 0: {                                                               // Optimization
+  case 0: { // Optimization
     OptHomParameters p;
-    p.nbLayers = nbLayers;
-    p.BARRIER_MIN = threshold_min;
-    p.BARRIER_MAX = threshold_max;
+    p.nbLayers = (int)o->value[2]->value();
+    p.BARRIER_MIN = o->value[1]->value();
+    p.BARRIER_MAX = o->value[8]->value();
     p.onlyVisible = onlyVisible;
     // change dim if no 3D elements are there
-    p.dim = GModel::current()->getDim() == 3 ? ( NE ? 3 : 2 ) :  GModel::current()->getDim();
-    printf("%d %d\n",NE,p.dim);
-    p.itMax = (int) o->value[3]->value();
-    p.optPassMax = (int) o->value[4]->value();
-    p.weightFixed =  o->value[5]->value();
-    p.weightFree =  o->value[6]->value();
-    p.distanceFactor =  o->value[7]->value();
+    p.dim = dim;
+    p.itMax = (int)o->value[3]->value();
+    p.optPassMax = (int)o->value[4]->value();
+    p.weight = o->value[5]->value();
+    p.distanceFactor = o->value[7]->value();
     p.fixBndNodes = (!o->CAD) || (o->choice[0]->value() == 0);
     p.strategy = o->choice[3]->value();
     p.maxAdaptBlob = o->value[9]->value();
-    p.adaptBlobLayerFact = (int) o->value[10]->value();
+    p.adaptBlobLayerFact = (int)o->value[10]->value();
     p.adaptBlobDistFact = o->value[11]->value();
     p.optPrimSurfMesh = false;
-    HighOrderMeshOptimizer(GModel::current(), p);
+    // HighOrderMeshOptimizer(GModel::current(), p);
+    HighOrderMeshOptimizerNew(GModel::current(), p);
     break;
   }
-  case 1: {                                                               // Elastic analogy
-    ElasticAnalogy(GModel::current(), threshold_min, onlyVisible);
+  case 1: { // Elastic analogy
+    ElasticAnalogy(GModel::current(), onlyVisible);
     break;
   }
-  case 2: {                                                               // Fast curving
+  case 2: { // Fast curving
     FastCurvingParameters p;
-    p.BARRIER_MIN = threshold_min;
-    p.BARRIER_MAX = threshold_max;
     p.onlyVisible = onlyVisible;
-    p.dim = GModel::current()->getDim() == 3 ? ( NE ? 3 : 2 ) :  GModel::current()->getDim();
-    p.distanceFactor =  o->value[7]->value();
+    p.dim = dim;
+    p.curveOuterBL = FastCurvingParameters::OUTER_CURVE;
+    p.thickness = false;
+    HighOrderMeshFastCurving(GModel::current(), p);
+    break;
+  }
+  case 3: { // Fast curving 2
+    FastCurvingParameters p;
+    p.onlyVisible = onlyVisible;
+    p.thickness = true;
+    if(dim == 3) {
+      p.dim = 2;
+      HighOrderMeshFastCurving(GModel::current(), p);
+    }
+    p.dim = dim;
     HighOrderMeshFastCurving(GModel::current(), p);
     break;
   }
@@ -182,7 +233,7 @@ static void highordertools_runopti_cb(Fl_Widget *w, void *data)
   Msg::Error("High-order mesh optimization requires the OPTHOM module");
 #endif
 
-  CTX::instance()->mesh.changed |= (ENT_LINE | ENT_SURFACE | ENT_VOLUME);
+  CTX::instance()->mesh.changed |= (ENT_CURVE | ENT_SURFACE | ENT_VOLUME);
   drawContext::global()->draw();
 }
 
@@ -194,48 +245,49 @@ highOrderToolsWindow::highOrderToolsWindow(int deltaFontSize)
   int width = 3 * IW + 4 * WB;
   int height = 24 * BH;
 
-  win = new paletteWindow
-    (width, height, CTX::instance()->nonModalWindows ? true : false, "High order tools");
+  win = new paletteWindow(width, height,
+                          CTX::instance()->nonModalWindows ? true : false,
+                          "High order tools");
   win->box(GMSH_WINDOW_BOX);
 
   int y = WB;
   int x = 2 * WB;
 
   box[0] = new Fl_Box(x, y, width - 4 * WB, BH);
-  box[0]->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
+  box[0]->align(FL_ALIGN_LEFT | FL_ALIGN_INSIDE);
 
   y += BH;
 
-  butt[1] = new Fl_Check_Button
-    (x, y, width - 4 * WB, BH, "Only apply high order tools to visible entities");
+  butt[1] =
+    new Fl_Check_Button(x, y, width - 4 * WB, BH,
+                        "Only apply high order tools to visible entities");
   butt[1]->type(FL_TOGGLE_BUTTON);
   butt[1]->value(1);
 
   y += BH;
 
-  butt[3] = new Fl_Check_Button
-    (x, y, width - 4 * WB, BH, "Show detailed log messages");
+  butt[3] =
+    new Fl_Check_Button(x, y, width - 4 * WB, BH, "Show detailed log messages");
   butt[3]->type(FL_TOGGLE_BUTTON);
-  butt[3]->value(1);
+  butt[3]->value(0);
 
   {
     y += BH / 2;
-    Fl_Box* b = new Fl_Box(x, y + BH - WB, width - 4 * WB, 2);
+    Fl_Box *b = new Fl_Box(x, y + BH - WB, width - 4 * WB, 2);
     b->box(FL_ENGRAVED_FRAME);
     b->labeltype(FL_NO_LABEL);
   }
 
   {
     y += BH;
-    Fl_Box *b = new Fl_Box
-      (x - WB, y, width, BH, "1. Generation of high order vertices");
+    Fl_Box *b =
+      new Fl_Box(x - WB, y, width, BH, "1. Generation of high order nodes");
     b->align(FL_ALIGN_LEFT | FL_ALIGN_INSIDE);
   }
 
   y += BH;
 
-  value[0] = new Fl_Value_Input
-    (x, y, IW, BH, "Polynomial order");
+  value[0] = new Fl_Value_Input(x, y, IW, BH, "Polynomial order");
   value[0]->minimum(1);
   value[0]->maximum(10);
   value[0]->step(1);
@@ -244,51 +296,62 @@ highOrderToolsWindow::highOrderToolsWindow(int deltaFontSize)
 
   y += BH;
 
-  butt[0] = new Fl_Check_Button
-    (x, y, width - 4 * WB, BH, "Use incomplete elements");
+  butt[0] = new Fl_Check_Button(x, y, width - 4 * WB, BH,
+                                "Generate incomplete elements");
   butt[0]->type(FL_TOGGLE_BUTTON);
   butt[0]->value(!complete);
-  butt[0]->callback(change_completeness_cb);
+  // AJ: This is bad a design. Add a proper Fl_Button to toggle between
+  // completeness if needed. Plus: only 2d version implemented for now
+  //  butt[0]->callback(change_completeness_cb);
 
   y += BH;
 
-  butt[2] = new Fl_Check_Button
-    (x, y, width - 4 * WB, BH, "Generate curvilinear elements");
+  butt[2] = new Fl_Check_Button(x, y, width - 4 * WB, BH,
+                                "Use CAD model to curve mesh");
   butt[2]->type(FL_TOGGLE_BUTTON);
   butt[2]->value(1);
 
   y += BH;
 
-  push[0] = new Fl_Button
-    (width - BB - 2 * WB, y, BB, BH, "Generate");
+  push[0] = new Fl_Button(width - BB - 2 * WB, y, BB, BH, "Generate");
   push[0]->callback(highordertools_runp_cb);
 
   {
     y += BH / 2;
-    Fl_Box* b = new Fl_Box(x, y + BH - WB, width - 4 * WB, 2);
+    Fl_Box *b = new Fl_Box(x, y + BH - WB, width - 4 * WB, 2);
     b->box(FL_ENGRAVED_FRAME);
     b->labeltype(FL_NO_LABEL);
   }
 
   {
     y += BH;
-    Fl_Box *b = new Fl_Box
-      (x - WB, y, width, BH, "2. Optimization of high order elements");
+    Fl_Box *b = new Fl_Box(x - WB, y, width, BH,
+                           "2. Regularization of high order elements");
     b->align(FL_ALIGN_LEFT | FL_ALIGN_INSIDE);
   }
 
   y += BH;
+  static Fl_Menu_Item menu_method[] = {
+    {"Optimization", 0, 0, 0},
+    {"Elastic Analogy", 0, 0, 0},
+    {"Fast Curving", 0, 0, 0},
+    {"Curve boundary layers (3D is experimental)", 0, 0, 0},
+    {0}};
+  choice[2] = new Fl_Choice(x, y, IW, BH, "Algorithm");
+  choice[2]->align(FL_ALIGN_RIGHT);
+  choice[2]->menu(menu_method);
+  choice[2]->callback(chooseopti_cb);
 
-  value[1] = new Fl_Value_Input
-    (x, y, IW/2.0, BH);
+  y += BH;
+  value[1] = new Fl_Value_Input(x, y, IW / 2.0, BH);
   value[1]->minimum(0);
   value[1]->maximum(1);
   value[1]->step(.01);
   value[1]->align(FL_ALIGN_RIGHT);
   value[1]->value(CTX::instance()->mesh.hoThresholdMin);
 
-  value[8] = new Fl_Value_Input
-    (x+IW/2.0,y, IW/2.0, BH, "Target Jacobian range");
+  value[8] =
+    new Fl_Value_Input(x + IW / 2.0, y, IW / 2.0, BH, "Target Jacobian range");
   value[8]->minimum(1);
   value[8]->maximum(10);
   value[8]->step(.01);
@@ -296,8 +359,7 @@ highOrderToolsWindow::highOrderToolsWindow(int deltaFontSize)
   value[8]->value(CTX::instance()->mesh.hoThresholdMax);
 
   y += BH;
-  value[2] = new Fl_Value_Input
-    (x, y, IW, BH, "Number of layers");
+  value[2] = new Fl_Value_Input(x, y, IW, BH, "Number of layers");
   value[2]->minimum(1);
   value[2]->maximum(20);
   value[2]->step(1);
@@ -305,53 +367,27 @@ highOrderToolsWindow::highOrderToolsWindow(int deltaFontSize)
   value[2]->value(CTX::instance()->mesh.hoNLayers);
 
   y += BH;
-  value[7] = new Fl_Value_Input
-    (x, y, IW, BH, "Distance factor");
+  value[7] = new Fl_Value_Input(x, y, IW, BH, "Distance factor");
   value[7]->minimum(1);
   value[7]->maximum(20000);
   value[7]->step(1);
   value[7]->align(FL_ALIGN_RIGHT);
   value[7]->value(12);
 
-  static Fl_Menu_Item menu_method[] = {
-    {"Optimization", 0, 0, 0},
-    {"Elastic Analogy", 0, 0, 0},
-    {"Fast Curving", 0, 0, 0},
-    {0}
-  };
-
   y += BH;
-  choice[2] = new Fl_Choice
-    (x, y, IW, BH, "Algorithm");
-  choice[2]->align(FL_ALIGN_RIGHT);
-  choice[2]->menu(menu_method);
-  choice[2]->callback(chooseopti_cb);
-
   static Fl_Menu_Item menu_objf[] = {
-    {"Fixed", 0, 0, 0},
-    {"Free", 0, 0, 0},
-    {0}
-  };
-
-  y += BH;
-  choice[0] = new Fl_Choice
-    (x, y, IW, BH, "Boundary vertices");
+    {"Fixed", 0, 0, 0}, {"Free", 0, 0, 0}, {0}};
+  choice[0] = new Fl_Choice(x, y, IW, BH, "Boundary nodes");
   choice[0]->menu(menu_objf);
   choice[0]->align(FL_ALIGN_RIGHT);
 
   y += BH;
-  value[5] = new Fl_Value_Input
-    (x, y, IW/2, BH);
+  value[5] = new Fl_Value_Input(x, y, IW, BH, "Weight on node displacement");
   value[5]->align(FL_ALIGN_RIGHT);
-  value[5]->value(1000.);
-  value[6] = new Fl_Value_Input
-    (x+IW/2,y, IW/2, BH, "W fixed / W free");
-  value[6]->align(FL_ALIGN_RIGHT);
-  value[6]->value(1.);
+  value[5]->value(1.);
 
   y += BH;
-  value[3] = new Fl_Value_Input
-    (x, y, IW, BH, "Maximum number of iterations");
+  value[3] = new Fl_Value_Input(x, y, IW, BH, "Maximum number of iterations");
   value[3]->minimum(1);
   value[3]->maximum(10000);
   value[3]->step(10);
@@ -359,30 +395,27 @@ highOrderToolsWindow::highOrderToolsWindow(int deltaFontSize)
   value[3]->value(300);
 
   y += BH;
-  value[4] = new Fl_Value_Input
-    (x, y, IW, BH, "Max. number of optimization passes");
+  value[4] = new Fl_Value_Input(x, y, IW, BH, "Max. number of barrier updates");
   value[4]->minimum(1);
   value[4]->maximum(100);
   value[4]->step(1);
   value[4]->align(FL_ALIGN_RIGHT);
   value[4]->value(50);
 
-  static Fl_Menu_Item menu_strategy[] = {
-    {"Connected blobs", 0, 0, 0},
-    {"Adaptive one-by-one", 0, 0, 0},
-    {0}
-  };
+  static Fl_Menu_Item menu_strategy[] = {{"Disjoint strong", 0, 0, 0},
+                                         {"Adaptive one-by-one", 0, 0, 0},
+                                         {"Disjoint weak", 0, 0, 0},
+                                         {0}};
 
   y += BH;
-  choice[3] = new Fl_Choice
-    (x, y, IW, BH, "Strategy");
+  choice[3] = new Fl_Choice(x, y, IW, BH, "Strategy");
   choice[3]->menu(menu_strategy);
   choice[3]->align(FL_ALIGN_RIGHT);
   choice[3]->callback(chooseopti_strategy);
 
   y += BH;
-  value[9] = new Fl_Value_Input
-    (x, y, IW, BH, "Max. number of blob adaptation iter.");
+  value[9] =
+    new Fl_Value_Input(x, y, IW, BH, "Max. number of patch adaptation iter.");
   value[9]->minimum(1);
   value[9]->maximum(100);
   value[9]->step(1);
@@ -391,8 +424,7 @@ highOrderToolsWindow::highOrderToolsWindow(int deltaFontSize)
   value[9]->deactivate();
 
   y += BH;
-  value[10] = new Fl_Value_Input
-    (x, y, IW, BH, "Num. layer adaptation factor");
+  value[10] = new Fl_Value_Input(x, y, IW, BH, "Num. layer adaptation factor");
   value[10]->align(FL_ALIGN_RIGHT);
   value[10]->minimum(1);
   value[10]->maximum(100);
@@ -401,21 +433,20 @@ highOrderToolsWindow::highOrderToolsWindow(int deltaFontSize)
   value[10]->deactivate();
 
   y += BH;
-  value[11] = new Fl_Value_Input
-    (x, y, IW, BH, "Distance adaptation factor");
+  value[11] = new Fl_Value_Input(x, y, IW, BH, "Distance adaptation factor");
   value[11]->align(FL_ALIGN_RIGHT);
   value[11]->minimum(1.);
   value[11]->maximum(100.);
   value[11]->value(2.);
   value[11]->deactivate();
 
-  y += 1.5*BH;
-  push[1] = new Fl_Button
-    (width - BB - 2 * WB, y, BB, BH, "Optimize");
+  y += 1.5 * BH;
+  push[1] = new Fl_Button(width - BB - 2 * WB, y, BB, BH, "Regularize");
   push[1]->callback(highordertools_runopti_cb);
 
   // win->resizable(o);
-  win->position(CTX::instance()->hotPosition[0], CTX::instance()->hotPosition[1]);
+  win->position(CTX::instance()->hotPosition[0],
+                CTX::instance()->hotPosition[1]);
   win->end();
   FL_NORMAL_SIZE += deltaFontSize;
 }
@@ -429,7 +460,7 @@ void highOrderToolsWindow::show(bool redrawOnly)
   else {
     value[0]->value(meshOrder);
     butt[0]->value(!complete);
-    if (CAD) {
+    if(CAD) {
       box[0]->label("CAD model is available");
       choice[0]->value(1);
     }
@@ -445,7 +476,7 @@ void highordertools_cb(Fl_Widget *w, void *data)
 {
   // get the visibility info from the model, and update the browser
   // accordingly
-  const char *str = (const char*)data;
+  const char *str = (const char *)data;
   if(str && !strcmp(str, "redraw_only"))
     FlGui::instance()->highordertools->show(true);
   else
