@@ -1,9 +1,9 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2018 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 //
-// Contributed by Matti Pellikka <matti.pellikka@tut.fi>.
+// Contributed by Matti Pellikka <matti.pellikka@microsoft.com>.
 
 #ifndef _HOMOLOGY_COMPUTATION_H_
 #define _HOMOLOGY_COMPUTATION_H_
@@ -13,15 +13,13 @@
 
 #if defined(HAVE_KBIPACK)
 
-extern "C"
-{
-  GMSH_Plugin *GMSH_RegisterHomologyComputationPlugin();
+extern "C" {
+GMSH_Plugin *GMSH_RegisterHomologyComputationPlugin();
 }
 
-class GMSH_HomologyComputationPlugin : public GMSH_PostPlugin
-{
- public:
-  GMSH_HomologyComputationPlugin(){}
+class GMSH_HomologyComputationPlugin : public GMSH_PostPlugin {
+public:
+  GMSH_HomologyComputationPlugin() {}
   std::string getName() const { return "HomologyComputation"; }
   std::string getShortHelp() const
   {
@@ -34,7 +32,7 @@ class GMSH_HomologyComputationPlugin : public GMSH_PostPlugin
   int getNbOptionsStr() const;
   StringXString *getOptionStr(int iopt);
   PView *execute(PView *);
-  bool parseStringOpt(int stringOpt, std::vector<int>& intList);
+  bool parseStringOpt(int stringOpt, std::vector<int> &intList);
 };
 
 #endif
